# Script for hinge, servo, motor, and corkscrew.
onStart {
  compoortopties = ["Seriele poort:", "Baudrate:"]
  # Standaard waarden!, deze kun je zelf aanpassen!
  standaard = ["COM8", "9600"]
  # Lijst met baudrates voor de dropdownlijst in het menu.
  baudratelijst = ["", "4800|9600|14400|19200|28800|38400|57600|115200"]
  # Geef het dialoogvenster weer voor vraag naar gegevens.
  serialsettings = UI.inputbox(compoortopties, standaard, baudratelijst, "Seriele poort instellingen:")
 
  if serialsettings != FALSE
    # Haal de instellingen uit het diagloogvenster en koppel deze aan een variabele.
    $comport = serialsettings[0]
    $baudrate = serialsettings[1].to_i
 
    begin
      # Open de seriele poort.
      @serial = Serial.new($comport, $baudrate)
    rescue RubySerial::Exception => e
      # Sluit de seriele poort als deze open staat.
      @serial.close if @serial
      # Reset/stop de simulatie.
      MSketchyPhysics3::SketchyPhysicsClient.physicsReset
      # Geef een bericht dat de seriele poort niet is gevonden.
      UI.messagebox("Seriele poort niet gevonden!")
    end
  else
    # Reset/stop de simulatie.
    MSketchyPhysics3::SketchyPhysicsClient.physicsReset
  end

  # Find connected joint
  @joint_parent = nil
  @joint = nil
  @joint_name = nil
  parent_joints = this.group.get_attribute("SPOBJ", "parentJoints").to_a
  parent_joints.each { |jname|
    if jname =~ /hinge|servo|motor|corkscrew/i
      @joint_name = jname
      break
    end
  }
  if @joint_name
    # Find joint group associated with joint name
    Sketchup.active_model.entities.each { |e|
      next unless e.is_a?(Sketchup::Group) || e.is_a?(Sketchup::ComponentInstance)
      if e.get_attribute('SPJOINT', 'name', '') == @joint_name
        @joint = e
      else
        found = false
        e.entities.each { |ec|
          if ec.is_a?(Sketchup::Group) && ec.get_attribute('SPJOINT', 'name', '') == @joint_name
            @joint_parent = e
            @joint = ec
            found = true
            break
          end
        }
        break if found
      end
    }
  end
  if @joint
    # Get min/max data associated with the joint
    min = @joint.get_attribute('SPJOINT', 'min', 0).to_f
    max = @joint.get_attribute('SPJOINT', 'max', 0).to_f
    @range = (max - min).degrees
    # Calculate starting angle
    if @joint_parent
      tra = @joint_parent.transformation * @joint.transformation
    else
      tra = @joint.transformation
    end
    @init_dir = tra.xaxis.transform(this.group.transformation.inverse)
    @init_angle = min.degrees
  end
}

onUpdate {
  if @joint
    # Calculate object angle and ratio
    if @joint_parent
      tra = @joint_parent.transformation * @joint.transformation
    else
      tra = @joint.transformation
    end
    dir = @init_dir.transform(this.group.transformation).transform(tra.inverse).normalize
    # Get theta from -PI to PI
    theta = -Math.acos(dir.x)
    theta = -theta if dir.y < 0
    ratio = @range.zero? ? 0 : (theta - @init_angle).to_f / @range

    # Soms kan de waarde kleiner zijn als nul (bij hard trekken aan het object)
    # zorg daarom dat deze niet onder de nul komt.
    ratio = 0 if ratio < 0
    # Geef de gegevens weer aan de linker zijde (log).
    logLine("COM poort: " + $comport.to_s)
    logLine("BAUD rate: " + $baudrate.to_s)
    logLine("")
    logLine("Hendel status: " + sprintf("%.2f", ratio).to_s)
    logLine("Hendel hoek: " + sprintf("%.2f",  theta.radians).to_s)
    logLine("")
    logLine("")
    logLine("")

    # Schrijf de data naar de seriele poort.
    @serial.write(sprintf("%.2f", ratio).to_s)
  end
}

# Wat moet er gebeuren als de de simulatie stopt?
onend{
  # Sluit de seriele poort als deze open staat.
  @serial.close if @serial
}